/// <reference types="node" />
import { IDebugger } from 'debug';
import { Certificate, JWK, KeyAndCert, KeystoreReader, PrivateKey } from './contracts';
export declare class Keystore {
    protected debug: IDebugger;
    protected signingKeyPassphrases: {
        [key: string]: string;
    };
    protected keystoreReader: KeystoreReader;
    /** The current signing key ID */
    protected currentSigningKeyId: string | null;
    /** Cache for the private key and certificate data */
    protected keys: Map<string, KeyAndCert>;
    protected interval?: NodeJS.Timeout;
    /**
     * Creates a new keystore service
     *
     * @param debugNamePrefix Name prefix used for the debug module
     * @param signingKeyPassphrases Stores passphrases for each signing keys. Key: key id, value: passphrase
     * @param keystoreReader Keystore reader callback (sync or async)
     */
    constructor(signingKeyPassphrases: {
        [key: string]: string;
    }, keystoreReader: KeystoreReader, debugNamePrefix?: string);
    /**
     * Start reader task
     *
     * @param refreshInterval Interval of the keystore refresh [millisec]
     */
    start(refreshInterval: number): Promise<void>;
    /**
     * Stop reader task
     */
    stop(): void;
    /**
     * Selects the current signing key id
     *
     * @param currentKeys The current keys
     *
     * @return The selected key id
     */
    selectCurrentSigningKeyId(currentKeys: Map<string, KeyAndCert>): string | null;
    /**
     * Returns the ID of the signing key that has to be used for signing
     *
     * @return The ID of the key that has to be used for signing
     */
    getCurrentSigningKeyId(): string | null;
    /**
     * Returns the passphrase for the given private key
     *
     * @param id The id of the private key
     *
     * @return The passphrase for the private key
     */
    getPrivateKeyPassphrase(id: string): string;
    /**
     * Returns the private key with the given id
     * Throws PrivateKeyNotFoundError if the certificate is not found in the store
     *
     * @param id The private key id
     *
     * @return Promise to the private key {key(PEM), passphrase}
     */
    getPrivateKey(id: string): Promise<PrivateKey>;
    /**
     * Returns the certificate with the given id
     * Throws CertificateNotFoundError if the certificate is not found in the store
     *
     * @param id The certificate id
     *
     * @return Promise to the certificate (PEM)
     */
    getCertificate(id: string): Promise<Certificate>;
    /**
     * Returns all certificates in JWKS format
     *
     * @return An array of JWK objects
     */
    getAllCertificatesAsJWKS(): JWK[];
    protected keystoreReaderTask(): Promise<void>;
}
