"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeystoreReaderFs = void 0;
const debug_1 = __importDefault(require("debug"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const pem = __importStar(require("pem"));
const jsrsasign_1 = require("jsrsasign");
class KeystoreReaderFs {
    /**
     * Creates a keystore reader function
     *
     * @param  baseDir The keystore base directory
     * @param  debugNamePrefix Name prefix used for the debug module
     */
    constructor(baseDir, debugNamePrefix) {
        this.debug = debug_1.default(`${debugNamePrefix ? debugNamePrefix + ':' : ''}keystore:reader:fs`);
        this.baseDir = baseDir;
        const rootCACert = fs.readFileSync(path.join(baseDir, `rootCA.cert.pem`));
        this.debug('Root CA cert loaded: ' + rootCACert);
        this.caCert = rootCACert.toString();
        this.debug('CA store created');
        this.algMap = {
            'SHA256withECDSA': 'ES256',
            'SHA256withRSA': 'RS256',
        };
    }
    /**
     * Handles the specified key id, i.e. returns a promise that loads the appropriate
     * private key and certificate from files
     *
     * @param {Map.<KeyAndCert>} newKeys A Map of key and cert data objects
     * @param {string} keyId Key ID
     * @return {Promise} A Promise that is resolved when the private and key a certificate for
     *                   the given keyId is loaded into the newKeys Object
     */
    async handleKeyId(newKeys, keyId) {
        this.debug(`keyId: ${keyId}`);
        if (!newKeys.has(keyId)) {
            this.debug(`${keyId} not found in cache`);
            await this.readKeyAndCert(newKeys, keyId);
        }
        else {
            this.debug(`${keyId} already cached`);
        }
    }
    /**
     * Creates s combined set of promises that are used for verifying and reading certs and private keys
     *
     * @param  {type} newKeys {Map.<KeyAndCert>} newKeys A Map of key and cert data objects
     * @param  {type} keyId Key ID
     * @return {Promise} A promise which is resolved when both the certificate and the
     *                   private key is loaded for the given key id
     */
    async readKeyAndCert(newKeys, keyId) {
        const privKeyPem = fs.readFileSync(path.join(this.baseDir, `${keyId}.privkey.pem`));
        const certPem = fs.readFileSync(path.join(this.baseDir, `${keyId}.cert.pem`));
        this.debug(`Key id ${keyId}, privkey and cert read`);
        const cert = new jsrsasign_1.X509();
        cert.readCertPEM(certPem.toString());
        const verifiedCert = await this.verifyCertAsync(keyId, certPem.toString());
        const certStr = verifiedCert.toString();
        const publicKey = jsrsasign_1.KEYUTIL.getKey(certStr);
        const pubkeyJwk = jsrsasign_1.KEYUTIL.getJWKFromKey(publicKey);
        pubkeyJwk.kid = keyId;
        pubkeyJwk.alg = this.algMap[cert.getSignatureAlgorithmField()];
        pubkeyJwk.use = 'sig';
        const x5c = jsrsasign_1.hextob64(jsrsasign_1.pemtohex(certStr));
        pubkeyJwk.x5c = [x5c];
        this.debug(`Key id ${keyId}, key alg: ${cert.getSignatureAlgorithmField()}`);
        newKeys.set(keyId, {
            timestamp: Math.floor(Date.now() / 1000),
            cert: certStr,
            keyAlg: this.algMap[cert.getSignatureAlgorithmField()],
            privKey: privKeyPem.toString(),
            pubkeyJwk,
        });
    }
    /**
     * Verifies the given certificate if it is certified with the KKSZB CA
     *
     * @param {string} keyId Key id
     * @param {string} certPem The certificate in PEM format
     * @return {Promise} A promise which is resolved if the given certificate is trusted
     */
    async verifyCertAsync(keyId, certPem) {
        await new Promise((resolve, reject) => pem.verifySigningChain(certPem, [this.caCert], (error, result) => error ? reject(error) : resolve(result)));
        this.debug(`Key id ${keyId}, signing certificate verified`);
        return certPem;
    }
    /**
     * Reads all keys and certs from the keystore base directory
     *
     * @param {Object} keys A map containing the currently loaded keys
     * @return {Promise.<Map.<KeyAndCert>, Error>} Promise to an object containing the already existing and loaded keys
     */
    async readKeys(keys) {
        const newKeys = new Map(keys);
        const files = fs.readdirSync(this.baseDir);
        await Promise.all(files.map(async (file) => {
            const match = file.match(/([0-9]+)\.privkey.pem/);
            if (match) {
                await this.handleKeyId(newKeys, match[1]);
            }
        }));
        this.debug('All keys loaded');
        return newKeys;
    }
}
exports.KeystoreReaderFs = KeystoreReaderFs;
//# sourceMappingURL=KeystoreReaderFs.js.map